// DIBWnd.cpp : implementation file
//

#include "stdafx.h"
#include "ITBReader.h"
#include "LibraryConnection.h"
#include "PictureWnd.h"
#include "DIBWnd.h"
#include "PictureDlg.h"
#include "PicturePropertiesDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////
// CDIBWnd

CDIBWnd::CDIBWnd()
{
	m_pDIBInfo = NULL;
	m_bDownloading = false;
	m_bDownloadError = true; // Just a known value
	m_pPicDlg = NULL;
}

CDIBWnd::~CDIBWnd()
{
}


BEGIN_MESSAGE_MAP(CDIBWnd, CPictureWnd)
	//{{AFX_MSG_MAP(CDIBWnd)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CDIBWnd message handlers
BOOL CDIBWnd::Create(const PICTUREINFO& pi, CRect& rect, CWnd* pParentWnd, UINT nID)
{
	if (pi.lSmallWidth && pi.lSmallHeight)
		m_fScaleFactor = (float)pi.lSmallWidth / (float)pi.lSmallHeight;
	else
		m_fScaleFactor = DEFAULTSCALEFACTOR;

	// Set the data values
	m_dwLargeSize = pi.dwLargeSize;
	m_lLargeWidth = pi.lLargeWidth;
	m_lLargeHeight = pi.lLargeHeight;

	m_bDownloading = true;

	// Create the window
	if (!CPictureWnd::Create(pi, rect, pParentWnd, nID))
		return FALSE;

	// Request the picture data
	REQUESTINFO* pRequestInfo = new REQUESTINFO;
	if (!pRequestInfo) return FALSE;	//there's not enough memory to do anything
	
	pRequestInfo->strObjectName.Format(_T("/ITBLibrary.dll?GetSmallPicture?Book=%d&Page=%d&Index=%d"),
		pi.uBook, pi.nPage, pi.uIndex);
	pRequestInfo->lpszVerb = NULL;
	pRequestInfo->nReadFuncType = RFT_DIB;
	pRequestInfo->hwndRequester = GetSafeHwnd();
	pRequestInfo->uBook = pi.uBook;
	pRequestInfo->nPage = pi.nPage;

	AfxBeginThread(InternetRequestThreadProc, pRequestInfo);

	return TRUE;
}

BOOL CDIBWnd::DestroyWindow() 
{
	// if it's downloading a picture, wait until finish
	// because it need to receive the message, so hide it meanwhile.
	if (m_bDownloading) {
		ShowWindow(SW_HIDE);
		SetTimer(1, 1000 /*1seg*/, NULL);
	}

	if (m_pDIBInfo) {
		delete m_pDIBInfo;	// See the DIBINFO destructor
		m_pDIBInfo = NULL;
	}

	if (m_pPicDlg) {
		m_pPicDlg->SetIndependent();
		m_pPicDlg = NULL;
	}
	
	return CPictureWnd::DestroyWindow();
}

void CDIBWnd::OnDraw(CDC* pDC, const CRect& rect)
{
	if (m_bDownloading || m_bDownloadError) 
	{
		CSize sz;
		int x, y;
		TCHAR* pTxt = (m_bDownloading) ? 
			TEXT("Downloading..."):TEXT("Download error");

		// Since SetTextAlign is not supported, 
		// let's do it by ourself.
		sz = pDC->GetTextExtent(pTxt, _tcslen(pTxt));

		if (sz.cx >= rect.Width() - 2)
			x = rect.left + 2;
		else
			x = rect.left + (rect.Width() - sz.cx) / 2;

		if (sz.cy >= rect.Height() - 2)
			y = rect.top + 2;
		else
			y = rect.top + (rect.Height() - sz.cy) / 2;

		pDC->SetBkColor(RGB(255, 255, 255));
		pDC->Rectangle(rect);
		CRect clip(x, y, rect.right - 4, rect.bottom - 4);
		pDC->ExtTextOut(x, y, ETO_CLIPPED, clip, pTxt, NULL);
	}
	else if (m_pDIBInfo)
	{
		HDC dcMem = ::CreateCompatibleDC(pDC->GetSafeHdc());
		HBITMAP hbmOld = (HBITMAP) ::SelectObject(dcMem, 
							m_pDIBInfo->hDIBSection);

		::StretchBlt(pDC->GetSafeHdc(), 
			rect.left, rect.top, 
			rect.Width(), rect.Height(), 
			dcMem, 0, 0,
			m_pDIBInfo->lpBMIH->biWidth, 
			m_pDIBInfo->lpBMIH->biHeight, 
			SRCCOPY); 

		::SelectObject(dcMem, hbmOld);
		::DeleteDC(dcMem);

		CBrush brush(RGB(0, 0, 0));
		pDC->FrameRect(rect, &brush);
	}
}

void CDIBWnd::OnDownloadCompleted(WPARAM wParam, LPARAM lParam)
{
	REQUESTINFO* pRequestInfo = (REQUESTINFO*)lParam;

	if (wParam == TRUE) 
	{
		ASSERT(pRequestInfo->pBuffer);
		ASSERT(pRequestInfo->dwBufferSize > 0);
	
		if (m_pDIBInfo) delete m_pDIBInfo;
		m_pDIBInfo = (DIBINFO*) pRequestInfo->pBuffer;
		m_bDownloadError = false;
	}
	else
		m_bDownloadError = true;

	delete pRequestInfo;	
	m_bDownloading = false;

	Invalidate();
}

void CDIBWnd::OnPictureSelect()
{
	ASSERT(!m_pPicDlg);
	
	m_pPicDlg = new CPictureDlg();

	if (!m_pPicDlg) {
		AfxMessageBox(IDS_NOT_ENOUGH_MEMORY);
		return;
	}

	BOOL bRet;

	if (m_dwLargeSize != 0)
		bRet = m_pPicDlg->Create(
			this, 
			GetPictureBook(),
			GetPicturePage(), 
			GetPictureIndex(), 
			m_strName,
			true /* request large version */, 
			m_lLargeWidth, 
			m_lLargeHeight, 
			m_dwLargeSize);
	
	if (m_dwLargeSize == 0 || !bRet)
		m_pPicDlg->Create(
			this, 
			GetPictureBook(),
			GetPicturePage(), 
			GetPictureIndex(), 
			m_strName,
			false, 
			m_pDIBInfo->lpBMIH->biWidth, 
			m_pDIBInfo->lpBMIH->biHeight, 
			m_pDIBInfo->lpBMIH->biSizeImage);

	// If last create didn't work user probably is out of memory
	// and the OS will warn him or her.
}

void CDIBWnd::OnPictureUnselect()
{
	ASSERT(m_pPicDlg);

	m_pPicDlg->DestroyWindow();

	delete m_pPicDlg;

	m_pPicDlg = NULL;
}

void CDIBWnd::OnTimer(UINT nIDEvent) 
{
	if(!m_bDownloading)
		DestroyWindow();	//It is supposed it destroy the timer too.
	
	//CPictureWnd::OnTimer(nIDEvent);
}

void CDIBWnd::OnProperties()
{
	CPicturePropertiesDlg dlg;

	dlg.m_strFileType = _T("Bitmap");
	
	if (!m_bDownloadError && !m_bDownloading) {
		dlg.m_strSmallSize.Format(_T("%d bytes"), m_pDIBInfo->lpBMIH->biSizeImage);
		dlg.m_strSmallWidth.Format(_T("%d bytes"), m_pDIBInfo->lpBMIH->biWidth);
		dlg.m_strSmallHeight.Format(_T("%d bytes"), m_pDIBInfo->lpBMIH->biHeight);
	}

	dlg.m_strLargeSize.Format(_T("%d bytes"), m_dwLargeSize);
	dlg.m_strLargeWidth.Format(_T("%d bytes"),  m_lLargeWidth);
	dlg.m_strLargeHeight.Format(_T("%d bytes"),  m_lLargeHeight);
	
	dlg.DoModal();
}
