#if !defined(AFX_HTMLVIEW_H__1D56EE61_E46A_11D2_AD1A_00A04B070143__INCLUDED_)
#define AFX_HTMLVIEW_H__1D56EE61_E46A_11D2_AD1A_00A04B070143__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
// HTMLView.h : header file
//
#include <afxtempl.h>
#include "TokenInfo.h"

struct CHARPOS		// Character Position
{
	POSITION pTokenInfo;
	int		 nIndex;

	bool operator==(const CHARPOS& rhs) const
	{ 
		return (pTokenInfo == rhs.pTokenInfo &&
			nIndex == rhs.nIndex);
	}
	bool operator!=(const CHARPOS& rhs) const
	{
		return !operator==(rhs);
	}
};

struct RELCHARPOS	// Line Relative Character Position
{
	CHARPOS		charPos;
	POSITION	pLine;
	bool		bIsLineEnd;
	//CPoint		pt;
};

struct TEXTLINE
{
	CHARPOS		first;
	CHARPOS		last;
	int			nHeight;
	int			nTopOffset;
	int			nBottomOffset;

	TEXTLINE() {nTopOffset = nBottomOffset = 0;}
	int TotalHeight() {
		return nTopOffset + nHeight + nBottomOffset;
	}
};

struct TEXTSTYLE
{
	CString strFontFamily;
	int		nForegroundColor;
	int		nSize;
	int		nTopOffset;
	int		nBottomOffset;
};

struct HIGHLIGHT
{
	CHARPOS		start;
	CHARPOS		end;
	int			nFirstChar;
	int			nLastChar;
	COLORREF	color;
	CString		strText;
	HIGHLIGHT() {
		color = RGB(255, 255, 0);
	}
	HIGHLIGHT& operator=(const HIGHLIGHT& rhs)
	{
		// It must be specified because of the CString param
		start = rhs.start;
		end = rhs.end;
		nFirstChar = rhs.nFirstChar;
		nLastChar = rhs.nLastChar;
		color = rhs.color;
		strText = rhs.strText; // Perform a right copy
		return *this;
	}
};

typedef CList<CTokenInfo, CTokenInfo&> TOKENLIST;
typedef CList<TEXTLINE, TEXTLINE&> PAGE;
typedef CList<HIGHLIGHT, HIGHLIGHT&> HIGHLIGHTLIST;

/////////////////////////////////////////////////////////////////////////////
// CHTMLView view

class CHTMLView : public CView
{
	static bool s_bIsClassRegistered;

protected:
	CHTMLView();           // protected constructor used by dynamic creation
	DECLARE_DYNCREATE(CHTMLView)

private:
	TOKENLIST		m_tokens;
	HIGHLIGHTLIST	m_hlList;
	RELCHARPOS		m_currPos;
	CPoint			m_caretPos;
	int				m_nCaretHeight;
	CHARPOS			m_selStart, m_selEnd;
	CPoint			m_ptSelStart, m_ptSelEnd;
	CRect			m_rcSelection;
	bool			m_bTextSelected;
	bool			m_bTextSelecting;
	bool			m_bModified;

	/*CDC*			m_pdcMemory;
	CBitmap*		m_pScreenBufferBmp;*/

// Styles
	TEXTSTYLE		m_styles[8];

// Attributes
public:
	CRect m_rcMargins;
	PAGE m_page;
	
// Operations
private:
	int CalcLineBreak(CTokenInfo& ti, int nLen, int cx, int& x);
	void PrepareFont(CFont*, int);
	void BreakLines();
	int GetNextWidth(CHARPOS* pCharPos);
	int GetPrevWidth(CHARPOS* pCharPos);
	CHARPOS GetNextCharPos(CHARPOS charPos);
	CHARPOS GetCharPos(int nAbsIndex);
	int GetAbsCharPos(const CHARPOS& cp);
	RELCHARPOS GetRelCharPos(const CHARPOS& cp);
	bool IsLineFirstChar(const TEXTLINE& line, const CHARPOS& cp);
	bool IsLineLastChar(const TEXTLINE& line, const CHARPOS& cp);
	bool IsLineEnd(const TEXTLINE& line, const CHARPOS& cp);
	void DrawLines(CDC* pDC);
	void DrawHighlightedFrames(CDC* pDC);
	void DrawSingleHighlightedFrame(const HIGHLIGHT& hl, CDC* pDC);
	void DrawSelection(CDC* pDC);
	bool GetCharFromPt(const CPoint& pt, RELCHARPOS* pRCP, CPoint* pPt = NULL);
	int GetTopOffset(int nStyle);
	int GetBottomOffset(int nStyle);
	int GetLineTotalHeight(int nLine);
	int CalcScrollPage();
	int GetScrollPage();
	void SetScrollPage();
	CTokenInfo* GetTokenInfoAtPt(const CPoint& pt);
		
	//void UpdateCaret();
	//void MoveCaretTo(const CPoint& pt, const RELCHARPOS& cp);

	int GetTokenInfoHeight(POSITION pos)
	{
		return m_tokens.GetAt(pos).m_nHeight;
	}

	int CalcTotalWidth(int* pWidths, int nLen)
	{
		ASSERT(nLen >= 0);
		int nTotalWidth;
		
		for (nTotalWidth = 0; nLen; nLen--)
			nTotalWidth += *pWidths++;

		return nTotalWidth;
	}

	bool IsTokenStyleAt(const CPoint& pt, int nStyle) //e.g. TI_HYPERLINK style
	{
		CTokenInfo* p = GetTokenInfoAtPt(pt);
		return (p && p->m_nStyle & nStyle) ? true:false;
	}

	bool IsSelectionInOrder()
	{
		return ((m_ptSelStart.y < m_ptSelEnd.y) ||
		(m_ptSelStart.y == m_ptSelEnd.y && m_ptSelStart.x <= m_ptSelEnd.x));
	}

public:
	virtual void OnLink(CString strData) {MessageBox(strData);}
	bool SetHTMLText(TCHAR* pBuffer, DWORD dwBufferSize);
	void PreparePage();
	
	void AddHighlight(int nFirstChar, int nLastChar, COLORREF nColor, LPCTSTR szText)
	{
		HIGHLIGHT hl;

		hl.start = GetCharPos(nFirstChar);
		hl.end = GetCharPos(nLastChar);
		hl.nFirstChar = nFirstChar;
		hl.nLastChar = nLastChar;
		hl.color = nColor;
		hl.strText = szText;

		m_hlList.AddTail(hl);
	}

	POSITION GetFirstHighlightPosition() {
		return m_hlList.GetHeadPosition();
	}

	HIGHLIGHT GetNextHighlight(POSITION& pos) {
		return m_hlList.GetNext(pos);
	}

	bool IsModified() { return m_bModified; }
	void SetModifiedFlag(bool bModified = true) {m_bModified = bModified;}

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CHTMLView)
	public:
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
	protected:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	virtual void OnInitialUpdate();     // first time after construct
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CHTMLView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	// Generated message map functions
	//{{AFX_MSG(CHTMLView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnHighlightText();
	afx_msg void OnUpdateHighlightText(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void DeleteContents();
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_HTMLVIEW_H__1D56EE61_E46A_11D2_AD1A_00A04B070143__INCLUDED_)
