// LibraryConnection.cpp : implementation file
//
#include "stdafx.h"
#include <WinInet.h>
#include "LibraryConnection.h"
#include "ErrorFunctions.h"

#ifdef _DEBUG
int g_RICount = 0;
#endif

/////////////////////////////////////////////////////////////////////////////
// Global variables for Internet connection
CString		g_strCurrentLibrary = TEXT("localhost");
CString		g_strUserName;
CString		g_strUserPassword;
UINT		g_uBookID = 0;
int			g_nCurrentPage = LIBRARY_PAGE;
bool		g_bIsConnected = false;
UINT		g_nIsReading = 0;
HINTERNET	g_hInet = NULL;
HINTERNET	g_hHttpSession = NULL;

TCHAR g_pchStatus[300];
HWND g_hStatusWnd = NULL;

/////////////////////////////////////////////////////////////////////////////
// Local function prototipes

bool ReadASCIIData(HINTERNET hHttpRequest, REQUESTINFO* pRI);
bool ReadDIBData(HINTERNET hHttpRequest, REQUESTINFO* pRI);
DWORD ComputeDIBSize(LPBITMAPINFOHEADER lpBMIH);

/////////////////////////////////////////////////////////////////////////////
// Function implementations

/* 
	A pice of advice from the InternetConnect() help:
		"For maximum efficiency, applications using the Gopher and HTTP 
	protocols should try to minimize calls to InternetConnect and 
	avoid calling this function for every transaction requested by 
	the user. One way to accomplish this is to keep a small cache 
	of handles returned from InternetConnect; when the user makes 
	a request to a previously accessed server, that session handle 
	is still available."
*/

void InternetDisconnect()
{
	if (!g_bIsConnected) return;

	//int nTimeOut = 5000000;
	//while (g_nIsReading && nTimeOut--); // Wait some time until the reader finish
	
	if (g_hHttpSession) {
		InternetCloseHandle(g_hHttpSession);
		g_hHttpSession = NULL;
	}
	
	if (g_hInet) {
		InternetCloseHandle(g_hInet);
		g_hInet = NULL;
	}

	g_bIsConnected = false;

/*#ifdef _DEBUG
	if (nTimeOut == 0)
		DbgErrorMsg(_T("Disconnected because timeout"));
#endif*/
}

UINT InternetConnectionThreadProc(LPVOID pParam)
{
	if (g_bIsConnected) 
		InternetDisconnect();
	
	g_hInet = InternetOpen(TEXT("ITBReader"), INTERNET_OPEN_TYPE_DIRECT , NULL, NULL, 0);

	if (g_hInet == NULL) {
		DbgErrorMsg(_T("Failed to open an Internet Session"));
		return 1;
	}

	// Set the callback function
	//if (pParam)
	//	g_hStatusWnd = (HWND) pParam;
	//	InternetSetStatusCallback(g_hInet, OnWinInetCallBack);

	g_hHttpSession = InternetConnect(g_hInet, g_strCurrentLibrary, INTERNET_DEFAULT_HTTP_PORT,
		NULL, NULL, INTERNET_SERVICE_HTTP, 0, 0 /*1 callback data*/);

	if (g_hHttpSession == NULL) 
	{
		InternetCloseHandle(g_hInet);
		g_hInet = NULL;
		DbgErrorMsg(_T("Failed to connect"));
		return 1;
	}

	g_bIsConnected = true;

	return 0;
}

UINT InternetRequestThreadProc(LPVOID pParam)
{
	REQUESTINFO* pRI = (REQUESTINFO*) pParam;
	HINTERNET hHttpRequest = NULL;
	UINT uRetCode = IR_ERROR_DEFAULT;
	LPCTSTR lpszAcceptTypes[] = {TEXT("text/*"), TEXT("image/*"), NULL};

	g_nIsReading++;
	
	if (!g_bIsConnected || !g_hInet || !g_hHttpSession)
		goto RequestEnd;
	

	hHttpRequest = HttpOpenRequest(g_hHttpSession, pRI->lpszVerb /*NULL=>GET*/, 
		pRI->strObjectName, NULL /*HTTP/1.0*/, NULL, lpszAcceptTypes, 
		0, 0 /*2 callback data*/);

	if (hHttpRequest == NULL) {
		uRetCode = IR_ERROR_OPEN;
		DbgErrorMsg(_T("Failed to open an HTTP Request"));
		goto RequestEnd;
	}

	///////////////////Check for valid page/////////////////////
	if (IsInvalidPage(pRI->nPage, pRI->uBook))
		goto RequestEnd;	// default uRetCode
	/////////////////////////End Check//////////////////////////

	if (pRI->lpszVerb && _tcscmp(pRI->lpszVerb, _T("POST")) == 0) 
	{
		CString format;

		format.Format(_T("X-Text: %d,%s"), pRI->dwBufferSize, pRI->pBuffer);
		HttpAddRequestHeaders(hHttpRequest, format, -1, 0);

		if (!HttpSendRequest(hHttpRequest, NULL /*No headers*/, 0, 
			"\r\nNote Data\r\n"/*pRI->pBuffer*/, 13/*pRI->dwBufferSize*/)) 
		{
			DbgErrorMsg(_T("Failed to send an HTTP POST Request"));
			uRetCode = IR_ERROR_SEND_POST;
		}
		else {
			DWORD status, len = 4;
			BOOL bQRet = HttpQueryInfo(hHttpRequest, 
					HTTP_QUERY_STATUS_CODE | HTTP_QUERY_FLAG_NUMBER,
					&status, &len, NULL);
			
			if (bQRet == FALSE)
				uRetCode = IR_ERROR_QUERY_INFO;
			else if (status == 404)
				uRetCode = IR_ANSW_404;
			else if (status == 200)
				uRetCode = IR_OK;
			else
				uRetCode = IR_ERROR_ANSW_UNK;
		}

		goto RequestEnd;
	}
	else	// It assumes GET request
	{
		if (!HttpSendRequest(hHttpRequest, NULL, 0, NULL, 0)) 
		{
			DbgErrorMsg(_T("Failed to send an HTTP GET Request"));
			uRetCode = IR_ERROR_SEND_GET;
		}
		else {
			DWORD status, len = 4;
			BOOL bQRet = HttpQueryInfo(hHttpRequest, 
					HTTP_QUERY_STATUS_CODE | HTTP_QUERY_FLAG_NUMBER,
					&status, &len, NULL);
			
			if (bQRet == FALSE)
				uRetCode = IR_ERROR_QUERY_INFO;
			else if (status == 404)
				uRetCode = IR_ANSW_404;
			else if (status == 200) {
				// Choose the correct read function
				bool bRet;

				if (pRI->nReadFuncType == RFT_ASCII)
					bRet = ReadASCIIData(hHttpRequest, pRI);
				else if (pRI->nReadFuncType == RFT_DIB)
					bRet = ReadDIBData(hHttpRequest, pRI);
				else if (pRI->nReadFuncType == RFT_VOID)
					bRet = true;
				else
					bRet = false;

				uRetCode = (bRet) ? IR_OK:IR_ERROR_RFT;
			}
			else
				uRetCode = IR_ERROR_ANSW_UNK;
		}
		goto RequestEnd;
	}

RequestEnd:
	if (hHttpRequest)
		InternetCloseHandle(hHttpRequest);

	if (pRI->hwndRequester) {
		ASSERT(::IsWindow(pRI->hwndRequester));
		::SendMessage(pRI->hwndRequester, WM_REQUEST_COMPLETED, 
			(uRetCode == IR_OK) ? TRUE:FALSE, (long)pRI);
	}
	
	g_nIsReading--;

	return uRetCode;
}

bool ReadASCIIData(HINTERNET hHttpRequest, REQUESTINFO* pRI)
{
	DWORD dwBytesRead = 0, dwSize = HTMLBUFF;
	int i = 1;

	pRI->pBuffer = new BYTE[HTMLBUFF + 1];	// Caller should free it. Plus 1 for '\0'
	if (!pRI->pBuffer) return false;

	pRI->dwBufferSize = 0;

	do {
		if(!InternetReadFile(hHttpRequest, pRI->pBuffer + pRI->dwBufferSize, 
			dwSize, &dwBytesRead))
		{
			DbgErrorMsg(_T("Failed to read an ASCII Request"));
			delete[] pRI->pBuffer;
			return false;
		}

		///////////////////Check for valid page/////////////////////
		if (IsInvalidPage(pRI->nPage, pRI->uBook)) {
			delete[] pRI->pBuffer;
			return false;
		}
		/////////////////////////End Check//////////////////////////
		
		if (dwBytesRead != 0) {
			pRI->dwBufferSize += dwBytesRead;

			if (dwBytesRead == dwSize)
			{
				i++;
				BYTE* pTmp = new BYTE[HTMLBUFF * i + 1];

				if (!pTmp) {
					delete[] pRI->pBuffer;
					return false;
				}

				memcpy(pTmp, pRI->pBuffer, pRI->dwBufferSize);
				delete[] pRI->pBuffer;
				pRI->pBuffer = pTmp;
				dwSize = HTMLBUFF;
			}
			else // FIX ME: What I should do, this: dwSize -= dwBytesRead; or: break;?
				break;
		}
		else 
			break;
	} while (true);

	if (pRI->dwBufferSize > 0) {
		pRI->pBuffer[pRI->dwBufferSize] = '\0';
		return true;
	}
	else {
		delete[] pRI->pBuffer;
		return false;
	}
}

DWORD ComputeDIBSize(LPBITMAPINFOHEADER lpBMIH)
{
	ASSERT(lpBMIH->biSizeImage == 0);

	DWORD dwBytes = ((DWORD) lpBMIH->biWidth * lpBMIH->biBitCount) / 32;
	if(((DWORD) lpBMIH->biWidth * lpBMIH->biBitCount) % 32) {
		dwBytes++;
	}
	dwBytes *= 4;
	return (dwBytes * lpBMIH->biHeight); // no compression
}

bool ReadDIBData(HINTERNET hHttpRequest, REQUESTINFO* pRI)
{
	BITMAPFILEHEADER bmfh;
	DWORD dwBytesRead, dwBMISize;

	if(!InternetReadFile(hHttpRequest, &bmfh, sizeof(bmfh), 
		&dwBytesRead) || dwBytesRead != sizeof(bmfh)) 
	{
		DbgErrorMsg(_T("Failed to read an HTTP Request"));
		return false;
	}

	if(bmfh.bfType != 0x4d42) 
	{
		/*char* pBuffer = new char[100];
		memcpy(pBuffer, (char*) &bmfh, sizeof(bmfh));
		InternetReadFile(hHttpRequest, pBuffer + sizeof(bmfh), 
			99 - sizeof(bmfh), &dwBytesRead);
		pBuffer[sizeof(bmfh) + dwBytesRead] = '\0';
		CString str;
		str.Format(TEXT("%hs"), pBuffer);
		AfxMessageBox(str);*/
		
		return false;
	}

	dwBMISize = bmfh.bfOffBits - sizeof(BITMAPFILEHEADER);

	DIBINFO* pDIBInfo = new DIBINFO;

	pDIBInfo->lpBMIH = (LPBITMAPINFOHEADER) new char[dwBMISize];

	// Read info hdr & color table
	if(!InternetReadFile(hHttpRequest, pDIBInfo->lpBMIH, dwBMISize, 
		&dwBytesRead) || dwBMISize != dwBytesRead)
	{
		delete pDIBInfo;
		DbgErrorMsg(_T("Failed to read an HTTP Request"));
		return false;
	}

	///////////////////Check for valid page/////////////////////
	if (IsInvalidPage(pRI->nPage, pRI->uBook)) {
		delete pDIBInfo;
		return false;
	}
	/////////////////////////End Check//////////////////////////

	ASSERT(::IsWindow(pRI->hwndRequester));

	HDC hDC = ::GetDC(pRI->hwndRequester);

	pDIBInfo->hDIBSection = ::CreateDIBSection(hDC, 
		(LPBITMAPINFO) pDIBInfo->lpBMIH, DIB_RGB_COLORS,
		(LPVOID*) &pDIBInfo->lpImage, NULL, 0);
	
	if (!pDIBInfo->hDIBSection) {
		delete pDIBInfo;
		DbgErrorMsg(_T("Null DIB section"));
		return false;
	}
	
	::ReleaseDC(pRI->hwndRequester, hDC);

	// Complete DIB's size information if not present
	if (pDIBInfo->lpBMIH->biSizeImage == 0)
		pDIBInfo->lpBMIH->biSizeImage = ComputeDIBSize(pDIBInfo->lpBMIH);

	// Read image only
	dwBytesRead = 0;
	DWORD dwTotal = 0;
	do {
		if(!InternetReadFile(hHttpRequest, pDIBInfo->lpImage + dwTotal, 
			pDIBInfo->lpBMIH->biSizeImage - dwTotal, &dwBytesRead))
		{
			delete pDIBInfo;
			DbgErrorMsg(_T("Failed to read an HTTP Request"));
			return false;
		}
		dwTotal += dwBytesRead;
		
		///////////////////Check for valid page/////////////////////
		if (IsInvalidPage(pRI->nPage, pRI->uBook)) {
			delete pDIBInfo;
			return false;
		}
		/////////////////////////End Check//////////////////////////

	} while (dwBytesRead != 0 || 
		pDIBInfo->lpBMIH->biSizeImage - dwTotal > 0);

	if (pDIBInfo->lpBMIH->biSizeImage != dwTotal) {
		delete pDIBInfo;
		return false;
	}
		
	pRI->pBuffer = (BYTE*) pDIBInfo;
	pRI->dwBufferSize = pDIBInfo->lpBMIH->biSizeImage;
	
	return true;
}

/* Callback function prototipe (as I found it in wininet.h)
typedef VOID (CALLBACK * INTERNET_STATUS_CALLBACK)(
    HINTERNET hInternet, DWORD dwContext,
    DWORD dwInternetStatus, LPVOID lpvStatusInformation,
    DWORD dwStatusInformationLength);
*/

void CALLBACK OnWinInetCallBack(HINTERNET hInternet, DWORD dwContext,
    DWORD dwInternalStatus, LPVOID lpvStatusInformation,
    DWORD dwStatusInformationLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int errors[] = {10, 11, 20, 21, 30, 31, 40, 41, 42, 43, 50, 51, 60, 70, 100, 110, 0};
	TCHAR* text[] = { 
		TEXT("Resolving name"),
		TEXT("Name resolved"),       
		TEXT("Connecting to server"),
		TEXT("Connected to server"),  
		TEXT("Sending request"),
		TEXT("Request sent"),
		TEXT("Receiving response"),   
		TEXT("Response received"),    
		TEXT("Ctl response received"),
		TEXT("Prefetch"),
		TEXT("Closing connection"),
		TEXT("Connection closed"),    
		TEXT("Handle created"),       
		TEXT("Handle closing"),       
		TEXT("Request complete"),     
		TEXT("Redirect"),
		TEXT("Unknown") };
	int n;
/*  // demonstrates request cancellation 
	if(dwInternalStatus == INTERNET_STATUS_REQUEST_SENT) {
		AfxThrowInternetException(dwContext, 999);
	}
*/
	for(n = 0; errors[n] != 0; n++) {
		if(errors[n] == (int) dwInternalStatus) break;
	}
	
	//g_csStatus.Lock();
	  
		/*strcpy(g_pchStatus, text[n]);
		if(dwInternalStatus  == INTERNET_STATUS_RESOLVING_NAME || 
				dwInternalStatus == INTERNET_STATUS_NAME_RESOLVED) {
		strcat(g_pchStatus, "-");
		strcat(g_pchStatus, (char*) lpvStatusInformation);
		}*/ 

		_tcscpy(g_pchStatus, text[n]);
		if(dwInternalStatus  == INTERNET_STATUS_RESOLVING_NAME || 
			dwInternalStatus == INTERNET_STATUS_NAME_RESOLVED) 
		{
			_tcscat(g_pchStatus, TEXT("-"));
			_tcscat(g_pchStatus, (TCHAR*) lpvStatusInformation);
		}

		TRACE(TEXT("WININET STATUS: %s\n"), g_pchStatus);

	//g_csStatus.Unlock();
	
	// It's better post it
	//::PostMessage(g_hStatusWnd, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)g_pchStatus);
}