// PictureView.cpp : implementation file
//

#include "stdafx.h"
#include "ITBReader.h"
#include "PictureView.h"
#include "LibraryConnection.h"
#include "DIBWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define	INTERPICFACTOR	0.025

/////////////////////////////////////////////////////////////////////////////
// CPictureView

IMPLEMENT_DYNCREATE(CPictureView, CView)

CPictureView::CPictureView()
{
	m_nTopMargin = 5;
	m_nLeftMargin = 5;
	m_nRightMargin = 5;
	m_nInterPicGap = 5;
	
	m_picArray.SetSize(0, 3);	// Increase by 3

	// Warning: this structure always should be alloqued 
	// in the heap because it'll be a thread param.
	m_pRequestInfo = new REQUESTINFO;
}

CPictureView::~CPictureView()
{
	if (m_pRequestInfo) delete m_pRequestInfo;
}


BEGIN_MESSAGE_MAP(CPictureView, CView)
	//{{AFX_MSG_MAP(CPictureView)
	ON_WM_SIZE()
	ON_WM_VSCROLL()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_REQUEST_COMPLETED, OnRequestCompleted)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPictureView drawing

void CPictureView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	SetScrollRange(SB_VERT, 0, 0);
	SetScrollPos(SB_VERT, 0);
}

void CPictureView::OnDraw(CDC* pDC)
{

}

/////////////////////////////////////////////////////////////////////////////
// CPictureView diagnostics

#ifdef _DEBUG
void CPictureView::AssertValid() const
{
	CView::AssertValid();
}

void CPictureView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPictureView message handlers

BOOL CPictureView::PreCreateWindow(CREATESTRUCT& cs) 
{
	cs.style |= WS_CLIPCHILDREN;	// To avoid fliking
	
	return CView::PreCreateWindow(cs);
}

void CPictureView::OnSize(UINT nType, int cx, int cy) 
{
	int nMax = m_picArray.GetSize();
	
	if (cx >= 0 && nMax > 0)
	{
		CPictureWnd* pPicWnd;
		CRect rect;
		CSize totalSize(0, m_nTopMargin);

		pPicWnd = m_picArray[0];
		
		pPicWnd->GetWindowRect(rect);
		ScreenToClient(rect);

		rect.left = m_nLeftMargin;
		rect.right = cx - m_nRightMargin;

		// Recalc layout
		pPicWnd->RecalcShadow(rect.Width());
		//m_nInterPicGap = (int)(cx * INTERPICFACTOR);
		
		pPicWnd->ScaleMove(&rect);
		totalSize.cy += rect.bottom - rect.top;
		rect.top = rect.bottom + m_nInterPicGap;
		
		for (int i = 1; i < nMax; i++)
		{
			pPicWnd = m_picArray[i];
			pPicWnd->ScaleMove(&rect);
			totalSize.cy += m_nInterPicGap + rect.bottom - rect.top;
			rect.top = rect.bottom + m_nInterPicGap;
		}

		CString str;
		str.Format(TEXT("Scroll %d"), nMax);
		MessageBox(str);
		//SetScrollPage((totalSize.cy) ? cy / (totalSize.cy / nMax):0);
	}
	
	CView::OnSize(nType, cx, cy);
}

void CPictureView::Empty() 
{
	CPictureWnd* p;
	int nMax = m_picArray.GetSize();
	for (int i = 0; i < nMax; i++)
	{
		p = m_picArray[i];
		p->DestroyWindow();
		delete p;
	}
	m_picArray.RemoveAll();
	SetScrollRange(SB_VERT, 0, 0);
}

void CPictureView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	if (pHint == 0) return;	// 0 => MFC default update
	
	while(m_bRequestInProgress);
	m_bRequestInProgress = true;

	Empty();
	
	//m_pRequestInfo->strObjectName = TEXT("hormiga.bmp");
	m_pRequestInfo->strObjectName = TEXT("pic_header.txt"/*"pic_header?page=0"*/);
	m_pRequestInfo->lpszVerb = NULL;
	m_pRequestInfo->nReadFuncType = RFT_ASCII;
	m_pRequestInfo->hwndRequester = GetSafeHwnd();
	m_pRequestInfo->nPage = (int)lHint;
	pRequestInfo->uBook = ::GetCurrentBook();


	AfxBeginThread(InternetRequestThreadProc, m_pRequestInfo);
}

void CPictureView::AddPicture(const PICTUREINFO& pi, CRect& rect)
{
	CPictureWnd* pPicWnd;
	
	switch (pi.nType)
	{
		case OT_DIB:
			pPicWnd = new CDIBWnd();
			pPicWnd->Create(pi, rect, this, 300 + m_picArray.GetSize());
			rect.top = rect.bottom + m_nInterPicGap;
			break;
		default:
			MessageBox(TEXT("default add picture"));
			pPicWnd = NULL;
			break;
	}
	
	if (pPicWnd) {
		m_picArray.Add(pPicWnd);
	}
	
	/*CString str;
	str.Format(TEXT("nPage = %d, nIndex = %d, strName = %s, nType = %d, bHasNote = %c, nSmallSize = %d, nLargeSize = %d, nSmallWidth = %d, nSmallHeight = %d, nLargeWidth = %d, nLargeHeight = %d"),
		pi.nPage, pi.nIndex, pi.strName, pi.nType, (pi.bHasNote) ? 'Y':'N', pi.nSmallSize, pi.nLargeSize, pi.nSmallWidth, pi.nSmallHeight, pi.nLargeWidth, pi.nLargeHeight);
	MessageBox(str);*/
}

void CPictureView::AddErrorPicture(const PICTUREINFO& pi, CRect& rect)
{
	MessageBox(TEXT("AddErrorPicture"));
}

LRESULT CPictureView::OnRequestCompleted(WPARAM wParam, LPARAM lParam)
{
	if (wParam == FALSE || m_pRequestInfo->dwBufferSize <= 0) 
	{
		MessageBox(TEXT("Request error"));
		return 0L;
		/*ASSERT(m_pRequestInfo->pBuffer);
	
		if (m_pDIBInfo) delete m_pDIBInfo;
		m_pDIBInfo = (DIBINFO*) m_pRequestInfo->pBuffer;
		Invalidate();*/
	}

	//m_bRequestInProgress = false;

	// I'd need to use strtok but I can't because I used it 
	// in the ParsePicHeaderLine function, so I did my own
	// strtok funtion that it's even better for this case.

	
	ASSERT(m_pRequestInfo->pBuffer);

	char* p = (char*) m_pRequestInfo->pBuffer;
	char* nexttoken = p;
	PICTUREINFO pi;
	CRect rect;

	GetClientRect(rect);

	//m_nInterPicGap = (int)(rect.Width() * INTERPICFACTOR);
	
	rect.left = m_nLeftMargin;
	rect.top = m_nTopMargin;
	rect.right -= m_nRightMargin;

	pi.nPage = ::GetCurrentPage();

	/*CString str;
	str.Format(TEXT("%hs"), p);
	MessageBox(str);*/

	pi.nIndex = 0;

	while (*nexttoken != '\0')
	{
		if (*nexttoken == '\n' /* the token*/)
		{
			*nexttoken = '\0';
			
			pi.nIndex++;

			if (ParsePicHeaderLine(p, pi))
				AddPicture(pi, rect);
			else 
				AddErrorPicture(pi, rect);

			p = ++nexttoken;
		}
		else nexttoken++;
	}

	SetScrollRange(SB_VERT, 0, m_picArray.GetSize() - 1);

	return 0L;
}

bool CPictureView::ParsePicHeaderLine(char* szLine, PICTUREINFO& pi)
{
	char* p;

	if (!(p = strtok(szLine, ","))) return false;

	// Get name string
	pi.strName.Format(TEXT("%hs"), p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get type
	//strupr(p);
	if (strcmp(p, "DIB") == 0) pi.nType = OT_DIB;
	else if (strcmp(p, "AVI") == 0) pi.nType = OT_AVI;
	else if (strcmp(p, "WAV") == 0) pi.nType = OT_WAV;
	else if (strcmp(p, "OLE") == 0) pi.nType = OT_OLE;
	else if (strcmp(p, "JPG") == 0) pi.nType = OT_JPG;
	else if (strcmp(p, "WAV") == 0) pi.nType = OT_GIF;
	else pi.nType = OT_UNK;
	if (!(p = strtok(NULL, ","))) return false;

	// Get note status
	if (*p == 'y' || *p == 'Y') pi.bHasNote = true;
	else pi.bHasNote = false;
	if (!(p = strtok(NULL, ","))) return false;

	// Get the small version's size
	pi.nSmallSize = atoi(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the large version's size
	pi.nLargeSize = atoi(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the small picture version's width
	pi.nSmallWidth = atoi(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the small picture version's height
	pi.nSmallHeight = atoi(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the large picture version's width
	pi.nLargeWidth = atoi(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the large picture version's height
	pi.nLargeHeight = atoi(p);
	
	// It's the last param and I don't care if 
	// there are more things, so I return true.

	return true;
}

int CPictureView::GetPictureHeight(int nIndex)
{
	CRect rect;
	m_picArray[nIndex]->GetWindowRect(rect);
	return rect.Height();
}

void CPictureView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int nPixelInc = 0, nPosInc = 0;
	UINT nCurPos = GetScrollPos(SB_VERT);
	UINT nMax = m_picArray.GetSize() - 1;//GetScrollLimit(SB_VERT);
	CRect clientRect;
	GetClientRect(clientRect);

	
	switch (nSBCode) {
		case SB_LINEDOWN: 
			if (nCurPos < nMax) {
				nPosInc = 1;
				nPixelInc = -GetPictureHeight(nCurPos);
			}
			break;
		case SB_LINEUP:
			if (nCurPos > 0) {
				nPosInc = -1;
				nPixelInc = GetPictureHeight(nCurPos - 1);
			}
			break;
		case SB_PAGEDOWN: {
			UINT i = nCurPos;
			int y = 0;

			while (i < nMax) {
				y += GetPictureHeight(i);
				if (y + clientRect.top <= clientRect.bottom) {
					i++;
					nPixelInc = -y;
				}
				else break;
			}
			nPosInc = i - nCurPos;
			break;
		}
		case SB_PAGEUP: {
			int i = nCurPos;

			while (i > 0 && clientRect.bottom - nPixelInc > clientRect.top)
				nPixelInc += GetPictureHeight(--i);

			nPosInc = i - nCurPos;
			break;
		}
		case SB_THUMBTRACK:
			if (nCurPos < nPos) {
				while (nCurPos + nPosInc < nPos && nCurPos + nPosInc < nMax) 
				{
					nPixelInc -= GetPictureHeight(nCurPos + nPosInc);
					nPosInc++;
				}
			}
			else if (nCurPos > nPos){
				do {
					nPosInc--;
					nPixelInc += GetPictureHeight(nCurPos + nPosInc);
				} while (nCurPos + nPosInc > nPos && nCurPos + nPosInc > 0);
			
				/*nPosInc--;
				while (nCurPos + nPosInc >= nPos && nCurPos + nPosInc >= 0) {
					nPixelInc += GetPictureHeight(nCurPos + nPosInc);
					nPosInc--;
				}
				nPosInc++;*/
			}
			break;
		default:
			nPosInc = 0;
			break;
	}

	if (nPosInc)
	{	
		nPixelInc -= nPosInc * m_nInterPicGap;

		int nMaxIndex = m_picArray.GetSize();
		CWnd* p;
		CRect rect;
		for (int i = 0; i < nMaxIndex; i++)
		{
			p = m_picArray[i];
			p->GetWindowRect(rect);
			ScreenToClient(rect);
			rect.OffsetRect(0, nPixelInc);
			p->MoveWindow(rect);
		}

		SetScrollPos(SB_VERT, nCurPos + nPosInc);
		
		UpdateWindow();

		CView::OnVScroll(nSBCode, nPos, pScrollBar);
	}
}

void CPictureView::SetScrollPage(int nPage)
{
	SCROLLINFO si;

	si.cbSize = sizeof(SCROLLINFO); 
    si.fMask = SIF_PAGE; 
	si.nPage = nPage;
	SetScrollInfo(SB_VERT, &si);
}

BOOL CPictureView::DestroyWindow() 
{
	Empty();
	
	return CView::DestroyWindow();
}
