// PictureWnd.cpp : implementation file
//

#include "stdafx.h"
#include "ITBReader.h"
#include "LibraryConnection.h"
#include "PictureWnd.h"
#include "PictureDlg.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TOPBANDHEIGHT		3		// It is based on icon top side
#define TEXTTOPMARGIN		1
#define LARGEICONSIZE		32
#define SMALLICONSIZE		16

bool CPictureWnd::s_bIsClassRegistered = false;
int CPictureWnd::s_nShadowHeight = 5;
int CPictureWnd::s_nShadowWidth = 5;
int CPictureWnd::s_nBottomBandHeight = 14; // It should've been based on font height at run time
HICON CPictureWnd::s_hLargeNoteIcon = NULL;
HICON CPictureWnd::s_hSmallNoteIcon = NULL;
HICON CPictureWnd::s_hLargeActiveNoteIcon = NULL;
HICON CPictureWnd::s_hSmallActiveNoteIcon = NULL;
HFONT CPictureWnd::s_labelFont = NULL;

/////////////////////////////////////////////////////////////////////////////
// CPictureWnd

CPictureWnd::CPictureWnd() : m_picRect(0, 0, 0, 0)
{
	// Put known values for test pourpose
	m_fScaleFactor = DEFAULTSCALEFACTOR;
	m_bHasNote = true;
	m_bIsPressed = false;
	m_bIsNoteActive = false;
	m_bSmallNoteIcon = true;
}

CPictureWnd::CPictureWnd(CString& strName)
{
	m_strName = strName;

	//m_hBitmap = ::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DIB_SAMPLE));
}

CPictureWnd::~CPictureWnd()
{

}


BEGIN_MESSAGE_MAP(CPictureWnd, CWnd)
	//{{AFX_MSG_MAP(CPictureWnd)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_ADD_DELETE_NOTE, OnAddDeleteNote)
	ON_COMMAND(ID_SAVE_NOTE, OnSaveNote)
	ON_COMMAND(ID_GET_PICTURE_PROPERTIES, OnGetPictureProperties)
	ON_COMMAND(ID_COPY_PICTURE, OnCopyPicture)
	ON_MESSAGE(WM_REQUEST_COMPLETED, OnRequestCompleted)
	ON_MESSAGE(WM_CLOSE_PICTURE_DLG, OnClosePictureDlg)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CPictureWnd message handlers
BOOL CPictureWnd::PreCreateWindow(CREATESTRUCT& cs) 
{
	// The window class should be modified to set
	// hCursor whit NULL value. Its needed to change
	// mouse cursor to hand cursor when its over a picture.
	// In this way it won't be necesary to capture cursor.
	static const TCHAR szClassName[] = _T("PicWndClass");
	
	if (!s_bIsClassRegistered)
	{
		WNDCLASS wc;

		// Start setting with common view values
		wc.style = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
		wc.lpfnWndProc = AfxWndProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = 0;
		wc.hInstance = AfxGetInstanceHandle();
		wc.hCursor = NULL;
		wc.hbrBackground = NULL; // To avoid fliking
		wc.lpszMenuName = NULL;
		wc.lpszClassName = szClassName;
		wc.hIcon = NULL;

		// Register window class
		if (!AfxRegisterClass(&wc)) {
			TRACE0("Failed to register PicWndClass\n");
			return 0;
		}
		
		s_bIsClassRegistered = true;
	}

	// Say to the MFC to use this class
	cs.lpszClass = szClassName;
	
	return CWnd::PreCreateWindow(cs);
}

BOOL CPictureWnd::Create(const PICTUREINFO& pi, CRect& rect, CWnd* pParentWnd, UINT nID)
{
	CalcBottomPt(&rect);

	m_uBook = pi.uBook;
	m_nPage = pi.nPage;
	m_uIndex = pi.uIndex;
	m_strName = pi.strName;
	m_bHasNote = pi.bHasNote;
	
	// Create the window
	if (!CWnd::Create(NULL, NULL, /*WS_BORDER |*/ WS_CHILD | WS_VISIBLE, 
		rect, pParentWnd, nID, NULL))
		return FALSE;

	RecalcPictureRect();
	
	return TRUE;
}

LRESULT CPictureWnd::OnRequestCompleted(WPARAM wParam, LPARAM lParam)
{
	OnDownloadCompleted(wParam, lParam);
	
	return 0L;
}

void CPictureWnd::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	CRect crect;
	GetClientRect(crect);

	int nBottomBand = crect.bottom - s_nBottomBandHeight;

	if (m_bIsPressed)
	{
		// Draw top white band
		dc.FillSolidRect(0, 0, crect.right, 
			TOPBANDHEIGHT + s_nShadowHeight, RGB(255, 255, 255));

		// Draw left white band
		dc.FillSolidRect(0, TOPBANDHEIGHT + s_nShadowHeight, 
			s_nShadowWidth, 
			nBottomBand - TOPBANDHEIGHT - s_nShadowHeight, 
			RGB(255, 255, 255));
	}
	else
	{
		// Draw top white band
		dc.FillSolidRect(0, 0, crect.right, TOPBANDHEIGHT, RGB(255, 255, 255));

		// Draw shadow
			// Right shadow white space
		dc.FillSolidRect(crect.right - s_nShadowWidth, TOPBANDHEIGHT, 
			s_nShadowWidth, s_nShadowHeight, RGB(255, 255, 255));
			
			// Right shadow
		dc.FillSolidRect(
			crect.right - s_nShadowWidth, 
			TOPBANDHEIGHT + s_nShadowHeight, 
			s_nShadowWidth, 
			nBottomBand - TOPBANDHEIGHT - s_nShadowHeight, 
			RGB(128, 128, 128));

			// Bottom shadow white space
		dc.FillSolidRect(0, nBottomBand - s_nShadowHeight, 
			s_nShadowWidth, s_nShadowHeight, RGB(255, 255, 255));
			
			// Bottom shadow
		dc.FillSolidRect(s_nShadowWidth, nBottomBand - s_nShadowHeight, 
			crect.right - s_nShadowWidth * 2, s_nShadowHeight, RGB(128, 128, 128));
	}

	OnDraw(&dc, m_picRect);	// Call the derived class's OnDraw function

	if (m_bHasNote)	DrawNoteIcon(&dc);

	// Draw bottom band
	dc.FillSolidRect(0, nBottomBand, crect.right, 
		s_nBottomBandHeight, RGB(255, 255, 255));

	// Draw text
	crect.top = nBottomBand;

	CFont* pOldFont = (CFont*)dc.SelectStockObject(ANSI_VAR_FONT);
	
	dc.ExtTextOut(s_nShadowWidth, nBottomBand + TEXTTOPMARGIN, 
		ETO_CLIPPED, crect, m_strName, NULL);

	if (pOldFont) dc.SelectObject(pOldFont);
}

void CPictureWnd::CalcBottomPt(CRect* pRect)
{
	int cx = pRect->Width() - s_nShadowWidth;

	if (cx > 0)
		pRect->bottom = pRect->top + TOPBANDHEIGHT + 
			(int)(cx / m_fScaleFactor) + 
			s_nShadowHeight + s_nBottomBandHeight;
	else
		pRect->bottom = pRect->top;
}

void CPictureWnd::ScaleMove(CRect* pRect, BOOL bRepaint /*= TRUE*/)
{
	CalcBottomPt(pRect);

	MoveWindow(pRect, bRepaint);

	RecalcPictureRect();
}

void CPictureWnd::RecalcShadow(int nNewWidth)
{
	s_nShadowHeight = s_nShadowWidth = (int)sqrt(nNewWidth * 0.2);
}

void CPictureWnd::RecalcPictureRect()
{
	CRect crect;
	GetClientRect(crect);

	if (m_bIsPressed)
		m_picRect.SetRect(s_nShadowWidth, TOPBANDHEIGHT + s_nShadowHeight,
			crect.right, crect.bottom - s_nBottomBandHeight);
	else
		m_picRect.SetRect(0, TOPBANDHEIGHT, crect.right - s_nShadowWidth, 
			crect.bottom - s_nBottomBandHeight - s_nShadowHeight);

	// Compute note icon size
	m_bSmallNoteIcon = (m_picRect.Width() >= LARGEICONSIZE * 1.5 &&
		m_picRect.Height() >= LARGEICONSIZE * 1.5) ? false:true;
}

void CPictureWnd::GetNoteIconRect(CRect& rect)
{
	rect.top = m_picRect.top - TOPBANDHEIGHT;
	rect.right = m_picRect.right;
	
	if (m_bSmallNoteIcon)
	{
		rect.left = m_picRect.right - SMALLICONSIZE;	
		rect.bottom = rect.top + SMALLICONSIZE;
	}
	else
	{
		rect.left = m_picRect.right - LARGEICONSIZE;	
		rect.bottom = rect.top + LARGEICONSIZE;
	}
}

void CPictureWnd::DrawNoteIcon(CDC* pDC)
{
	HICON hIcon;
	int cx;

	/*LR_SHARED flag didn't worked*/

	if (m_bSmallNoteIcon && m_bIsNoteActive) {
		if (!s_hSmallActiveNoteIcon)
			s_hSmallActiveNoteIcon = (HICON)::LoadImage(
				AfxGetInstanceHandle(), 
				MAKEINTRESOURCE(IDI_SMALL_ACTIVE_CLIP), 
				IMAGE_ICON, 0, 0, 0);
		hIcon = s_hSmallActiveNoteIcon;
		cx = SMALLICONSIZE;
	}
	else if (m_bSmallNoteIcon && !m_bIsNoteActive) {
		if (!s_hSmallNoteIcon)
			s_hSmallNoteIcon = (HICON)::LoadImage(
				AfxGetInstanceHandle(), 
				MAKEINTRESOURCE(IDI_SMALL_CLIP), 
				IMAGE_ICON, 0, 0, 0);
		hIcon = s_hSmallNoteIcon;
		cx = SMALLICONSIZE;
	}
	else if (!m_bSmallNoteIcon && m_bIsNoteActive) {
		if (!s_hLargeActiveNoteIcon)
			s_hLargeActiveNoteIcon = (HICON)::LoadImage(
				AfxGetInstanceHandle(), 
				MAKEINTRESOURCE(IDI_ACTIVE_CLIP), 
				IMAGE_ICON, 0, 0, 0);
		hIcon = s_hLargeActiveNoteIcon;
		cx = LARGEICONSIZE;
	}
	else /*if (!m_bSmallNoteIcon && !m_bIsNoteActive)*/ {
		if (!s_hLargeNoteIcon)
			s_hLargeNoteIcon = (HICON)::LoadImage(
				AfxGetInstanceHandle(), 
				MAKEINTRESOURCE(IDI_CLIP), 
				IMAGE_ICON, 0, 0, 0);
		hIcon = s_hLargeNoteIcon;
		cx = LARGEICONSIZE;
	}
	
	// Draw note icon
	::DrawIconEx(pDC->GetSafeHdc(),
		m_picRect.right - cx, 
		(m_bIsPressed) ? s_nShadowHeight:0,
		hIcon, 0, 0, 0, NULL, DI_NORMAL);
}

void CPictureWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (m_bHasNote) 
	{
		int iconSz = (m_bSmallNoteIcon) ? SMALLICONSIZE:LARGEICONSIZE;
		CRect iconRect(
			m_picRect.right - iconSz, 
			m_picRect.top - TOPBANDHEIGHT,
			m_picRect.right,
			m_picRect.top + iconSz - TOPBANDHEIGHT);

		if (iconRect.PtInRect(point))
		{
			if(GetParent()->SendMessage(WM_NOTE_CLICK, (WPARAM)this, m_uIndex))
				SetNoteActive();

			return;
		}
	}
	
	if (m_picRect.PtInRect(point))
	{
		PushPicture(!m_bIsPressed);
	}
}

void CPictureWnd::PushPicture(bool bPush /*= true*/)
{
	if (bPush && !m_bIsPressed) {
		m_bIsPressed = true;
		OnPictureSelect();
	}
	else if (!bPush && m_bIsPressed) {
		m_bIsPressed = false;
		OnPictureUnselect();
	}
	else return;
	
	RecalcPictureRect();
	Invalidate();
}

void CPictureWnd::SetNoteActive(bool bState /*= true*/)
{
	ASSERT(m_bHasNote);

	m_bIsNoteActive = bState;
	CDC* pDC = GetDC();
	DrawNoteIcon(pDC);	//or InvalidateRect(iconRect);
	ReleaseDC(pDC);
}

void CPictureWnd::OnAddDeleteNote()
{
	if (m_bHasNote)
	{
		if (GetParent()->SendMessage(WM_DELETE_NOTE, 
			(WPARAM)this, m_uIndex))
		{
			m_bHasNote = false;
			CRect iconRect;
			GetNoteIconRect(iconRect);
			InvalidateRect(iconRect);
		}
	}		
	else
	{
		if (GetParent()->SendMessage(WM_ADD_NOTE, 
			(WPARAM)this, m_uIndex))
		{
			m_bHasNote = true;	
			SetNoteActive();
		}
	}
}

void CPictureWnd::OnSaveNote()
{
	AfxMessageBox(_T("This option is here just to make the popup look nicer. It will be implemented in ITBReader version 2"));
}

void CPictureWnd::OnCopyPicture()
{
	AfxMessageBox(_T("This option is here just to make the popup look nicer. It will be implemented in ITBReader version 2"));
}

void CPictureWnd::OnGetPictureProperties()
{
	OnProperties();
}

void CPictureWnd::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CMenu popup;

	if (!popup.CreatePopupMenu()) return;

	popup.AppendMenu(MF_STRING, ID_COPY_PICTURE, _T("&Copy"));
	popup.AppendMenu(MF_STRING, ID_SAVE_NOTE, _T("&Save"));
	popup.AppendMenu(MF_STRING, ID_GET_PICTURE_PROPERTIES, _T("&Properties"));

	popup.AppendMenu(MF_SEPARATOR);

	popup.AppendMenu(MF_STRING, ID_ADD_DELETE_NOTE, 
		(m_bHasNote) ? _T("&Delete note"):_T("&Add note"));

	ClientToScreen(&point);
	popup.TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y, this);
	
	CWnd::OnRButtonDown(nFlags, point);
}

void CPictureWnd::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_bHasNote) 
	{
		int iconSz = (m_bSmallNoteIcon) ? SMALLICONSIZE:LARGEICONSIZE;
		CRect iconRect(
			m_picRect.right - iconSz, 
			m_picRect.top - TOPBANDHEIGHT,
			m_picRect.right,
			m_picRect.top + iconSz - TOPBANDHEIGHT);

		if (iconRect.PtInRect(point))
		{
			::SetCursor(AfxGetApp()->LoadCursor(IDC_NOTE_HAND));
			return;
		}
	}
	
	if (m_picRect.PtInRect(point))
		::SetCursor(AfxGetApp()->LoadCursor(IDC_HAND));
	else
		::SetCursor(::LoadCursor(NULL, IDC_ARROW));
}

LRESULT CPictureWnd::OnClosePictureDlg(WPARAM wParam, LPARAM lParam)
{
	PushPicture(false);
	
	return 0L;
}
