#include "stdafx.h"
#include "afxinet.h"
#include <afxmt.h>
#include "WinInetUtility.h"


CString g_strLibraryName = TEXT("localhost");
CString g_strLibraryIP = TEXT("127.0.0.1");
CString g_strStatus = TEXT("NOT CONNECTED");
int g_nPort = 80;
HWND g_hwndStatus = NULL;

TCHAR g_pchStatus[];
CCriticalSection g_csStatus;

CCallbackInternetSession::CCallbackInternetSession( LPCTSTR pstrAgent, DWORD dwContext,
	 DWORD dwAccessType, LPCTSTR pstrProxyName, LPCTSTR pstrProxyBypass, DWORD dwFlags) :
	CInternetSession(pstrAgent, dwContext, dwAccessType, pstrProxyName, pstrProxyBypass, dwFlags)
{
	EnableStatusCallback();
}

void CCallbackInternetSession::OnStatusCallback(DWORD dwContext, DWORD dwInternalStatus,
		LPVOID lpvStatusInformation, DWORD dwStatusInformationLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	int errors[] = {10, 11, 20, 21, 30, 31, 40, 41, 42, 43, 50, 51, 60, 70, 100, 110, 0};
	char* text[] = { 
		"Resolving name",       	
		"Name resolved",       
		"Connecting to server",
		"Connected to server",  
		"Sending request",      
		"Request sent",         
		"Receiving response",   
		"Response received",    
		"Ctl response received",
		"Prefetch",             
		"Closing connection",   
		"Connection closed",    
		"Handle created",       
		"Handle closing",       
		"Request complete",     
		"Redirect",
		"Unknown" };
	int n;
/*  // demonstrates request cancellation 
	if(dwInternalStatus == INTERNET_STATUS_REQUEST_SENT) {
		AfxThrowInternetException(dwContext, 999);
	}
*/
	for(n = 0; errors[n] != 0; n++) {
		if(errors[n] == (int) dwInternalStatus) break;
	}
	g_csStatus.Lock();
	  strcpy(g_pchStatus, text[n]);
	  if(dwInternalStatus  == INTERNET_STATUS_RESOLVING_NAME || 
				dwInternalStatus == INTERNET_STATUS_NAME_RESOLVED) {
		strcat(g_pchStatus, "-");
		strcat(g_pchStatus, (char*) lpvStatusInformation);
	  }
	  TRACE("WININET STATUS: %s\n", g_pchStatus);
	g_csStatus.Unlock();
	// Post it to finish fast
	::PostMessage(g_hwndStatus, WM_SETTEXT, 0, (LPARAM)(LPCTSTR) g_pchStatus);
}

void LogInternetException(LPVOID pParam, CInternetException* pe)
{	// pParam holds the HWND for the destination window (in another thread)
	CString strGmt = CTime::GetCurrentTime().FormatGmt("%m/%d/%y %H:%M:% GMT");
	char text1[300], text2[100];
	wsprintf(text1, "CLIENT ERROR: WinInet error #%d -- %s\r\n   ",
		pe->m_dwError, (const char*) strGmt);
	pe->GetErrorMessage(text2, 99);
	strcat(text1, text2);
	if(pe->m_dwError == 12152) {
		strcat(text1, "  URL not found?\r\n");
	}
	::SendMessage((HWND) pParam, EM_SETSEL, (WPARAM) 65534, 65535);
	::SendMessage((HWND) pParam, EM_REPLACESEL, (WPARAM) 0, (LPARAM) text1);
}

UINT ClientWinInetThreadProc(LPVOID pParam)
{
	CCallbackInternetSession session;
	CHttpConnection* pConnection = NULL;
	CHttpFile* pFile1 = NULL;
	char* buffer = new char[MAXBUF];
	UINT nBytesRead = 0;
	DWORD dwStatus;
	try {
		// username/password doesn't work yet
		if(!g_strServerName.IsEmpty())
			pConnection = session.GetHttpConnection(
				g_strServerName, (INTERNET_PORT) g_nPort);
		else
			pConnection = session.GetHttpConnection(
				g_strServerIP, (INTERNET_PORT) g_nPort);
		
		pFile1 = pConnection->OpenRequest(1, g_strFile, NULL, 1, NULL, NULL, // GET request
				INTERNET_FLAG_KEEP_CONNECTION); // needed for NT Challenge/Response authentication
				// INTERNET_FLAG_RELOAD forces reload from the server (bypasses client's cache)

		pFile1->SendRequest();
		pFile1->QueryInfoStatusCode(dwStatus);

		TRACE("QueryInfoStatusCode = %d\n", dwStatus);
		
		nBytesRead = pFile1->Read(buffer, MAXBUF - 1);

		buffer[nBytesRead] = '\0'; // necessary for message box
		
		char temp[100];
		
		if(pFile1->Read(temp, 100) != 0) { // makes caching work if read complete
			AfxMessageBox("File overran buffer -- not cached");
		}
		::MessageBox(::GetTopWindow(::GetDesktopWindow()), buffer, "WININET CLIENT", MB_OK);
		// could use existing pFile1 to SendRequest again if we wanted to
	}
	catch(CInternetException* e) {
		LogInternetException(pParam, e);
		e->Delete();
	}
	
	// could call OpenRequest again on same connection if we wanted to 
	if(pFile1) delete pFile1; // does the close -- prints a warning
	if(pConnection) delete pConnection; // why does it print a warning?
	delete [] buffer;
	
	g_csStatus.Lock();
	  strcpy(g_pchStatus, ""); // problem with empty string. bug #9897
	g_csStatus.Unlock();
	
	session.Close();
	
	return 0;
}


UINT ClientWinInetThreadProc(LPVOID pParam)
{
	HINTERNET hInet, hHttpSession, hHttpRequest, hFile;
	LPCTSTR lpszAcceptTypes[] = {"text/*", NULL};
	char szBuffer[MAXBUF];
	long lBytesRead;

	hInet = InternetOpen("ITBReader", INTERNET_OPEN_TYPE_DIRECT , NULL, NULL, 0);

	if (hInet == NULL)
		AfxMessageBox(TEXT("Failed to open an Internet Connection"));

	hHttpSession = IntermetConnect(hInet, g_strLibraryName, INTERNET_DEFAULT_HTTP_PORT,
		NULL, NULL, INTERNET_SERVICE_HTTP, 0, 0);

	if (hHttpSession == NULL)
		AfxMessageBox(TEXT("Failed to open an Internet Session"));

	hHttpRequest = HttpOpenRequest(hHttpSession, NULL /*=>GET*/, TEXT("/custom"), 
		NULL /*HTTP/1.0*/, NULL, lpszAcceptTypes, 0, 0);

	if (hHttpRequest == NULL)
		AfxMessageBox(TEXT("Failed to open an HTTP Request"));

	hFile = HttpSendRequest(hHttpRequest, NULL /*No headers*/, 0, NULL, 0);

	if (hFile == NULL)
		AfxMessageBox(TEXT("Failed to send an HTTP Request"));

	if (InternetReadFile(hFile, szBuffer, MAXBUF - 1, &lBytesRead))
	{
		szBuffer[lBytesRead] = '\0';
		AfxMessageBox(szBuffer);
	}
	else
		AfxMessageBox(TEXT("Failed to read an HTTP Request"));

	InternetCloseHandle(hFile);
	InternetCloseHandle(hHttpRequest);
	InternetCloseHandle(hHttpSession);
	InternetCloseHandle(hInet);
}